
<div id="accordion" role="tablist" aria-multiselectable="true">

    {% for method in methods %}
        {% include 'modernrpc/doc_accordion_method.html' %}
    {% endfor %}

</div> <!-- /accordion -->
